/* definition module FoxParm */
/*
  Generic functions to handle most value and reference
  parameter situations in PLBs.

  Free, no responsibilities taken. 02/1993 Roland Kappeler
*/

#include <pro_ext.h>

/* ------------------------------------------------------ */

void ParmToVar (    Parameter *OneParm,   /* &parm->p [xxx]          */
                    int        ArrElmt,   /* array elmt nbr (from 1) */
                    void      *TheVar,    /* To-Var                  */
                    int        VarSize,   /* sizeof (TheVar)         */
                    char       Type);     /* Type expected           */
/*
  Moves a fox parameter to "TheVar". Types: "C", "I", "N" and "L".
  The routine assumes that the caller knows the type expected.
  Reference parameters are loaded as required.
  If ArrElmt is not zero, a one-dimensional array is assumed.

  The caller determines the move size with "VarSize". On 'I' or 'N'
  types char, int /short, long, and double ('N') can be passed
  (according to the expected range, of course).

  'I' and 'N' types are converted into the type requested by the
  "Type" parameter. Other type differences result in an error.
*/
/* ------------------------------------------------------ */

void VarToParm (    Parameter *OneParm,   /* &parm->p [xxx]          */
                    int        ArrElmt,   /* array elmt nbr (from 1) */
                    void      *TheVar,    /* From-Var                */
                    int        VarSize,   /* sizeof (TheVar)         */
                    char       Type);     /* Type of ret. value      */
/*
  Returns a variable to a reference parameter.
  Types: "C", "I", "N" and "L".
  The parameter block element determines the type.
  If ArrElmt is not zero, a one-dimensional array is assumed.
  The caller determines the move size with "VarSize".

  Types 'N' and 'I' are returned according to the "Type" parameter,
  thereby possibly changing the type of the original Fox variable.
*/
/* ------------------------------------------------------------ */

/*

  Usage examples:

1:

  FoxInfo {{"VALPARM", ValParm, "I,C,N"}};

  Parameters expected: Int, String, Real

  void FAR ValParm (ParamBlk FAR *parm)

  {
  long   Long;
  char   String [80];
  double Real;

  /* Fetch parameters: */
    ParmToVar (&parm->p [0], 0, &Long,   sizeof (long)  , 'I');
    ParmToVar (&parm->p [1], 0, &String, sizeof (String), 'C');
    ParmToVar (&parm->p [2], 0, &Real,   sizeof (Real),   'N');
  }

2:

  FoxInfo {{"REFPARM", RefParm, "R,R,R"}};

  Parameters expected: @Int, @String, @Real

  void FAR RefParm (ParamBlk FAR *parm)

  {
  long   Long;
  char   String [80];
  double Real;

  /* Fetch parameters: */
    ParmToVar (&parm->p [0], 0, &Long,   sizeof (long),   'I');
    ParmToVar (&parm->p [1], 0, &String, sizeof (String), 'C');
    ParmToVar (&parm->p [2], 0, &Real,   sizeof (Real),   'N');

  /* Return values to Fox */
    VarToParm (&parm->p [0], 0, &Long,   sizeof (long),   'I');
    VarToParm (&parm->p [1], 0, &String, sizeof (String), 'C');
    VarToParm (&parm->p [2], 0, &Real,   sizeof (Real),   'N');
  }

/* end FoxParm. */